@@ -1,18 +1,22 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 
 import sys
 import os
 import gettext
 import glob
-from gi.repository import GLib, Gtk, Gio, CMenu, GdkPixbuf
 from optparse import OptionParser
 import shutil
 
-sys.path.insert(0,'/usr/share/cinnamon/cinnamon-menu-editor')
+import gi
+gi.require_version("Gtk", "3.0")
+gi.require_version("CMenu", "3.0")
+from gi.repository import GLib, Gtk, Gio, CMenu, GdkPixbuf
+
+sys.path.insert(0, '/usr/share/cinnamon/cinnamon-menu-editor')
 from cme import util
 
-sys.path.insert(0,'/usr/share/cinnamon/cinnamon-settings')
-from bin import XletSettingsWidgets
+sys.path.insert(0, '/usr/share/cinnamon/cinnamon-settings/bin')
+import JsonSettingsWidgets
 
 # i18n
 gettext.install("cinnamon", "/usr/share/locale")
@@ -24,9 +28,12 @@ PANEL_LAUNCHER_PATH = os.path.join(home, ".cinnamon", "panel-launchers")
 
 EXTENSIONS = (".png", ".xpm", ".svg")
 
+DEFAULT_ICON_NAME = "cinnamon-panel-launcher"
+
 def escape_space(string):
     return string.replace(" ", "\ ")
 
+
 def try_icon_name(filename):
     # Detect if the user picked an icon, and make
     # it into an icon name.
@@ -52,6 +59,7 @@ def try_icon_name(filename):
 
     return parts[3]
 
+
 def get_icon_string(image):
     filename = image._file
     if filename is not None:
@@ -59,21 +67,28 @@ def get_icon_string(image):
 
     return image._icon_name
 
+
 def strip_extensions(icon):
     if icon.endswith(EXTENSIONS):
         return icon[:-4]
     else:
         return icon
 
+
 def set_icon_string(image, icon):
     if GLib.path_is_absolute(icon):
-        image._file = icon
-        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(icon, 64, 64)
-        if pixbuf is not None:
-            image.set_from_pixbuf(pixbuf)
+        if os.path.isfile(icon):
+            image._file = icon
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(icon, 64, 64)
+            if pixbuf is not None:
+                image.set_from_pixbuf(pixbuf)
+        else:
+            image._icon_name = DEFAULT_ICON_NAME
+            image.set_from_icon_name(DEFAULT_ICON_NAME, Gtk.IconSize.BUTTON)
     else:
         image._icon_name = strip_extensions(icon)
-        image.set_from_icon_name (strip_extensions (icon), Gtk.IconSize.BUTTON)
+        image.set_from_icon_name(strip_extensions(icon), Gtk.IconSize.BUTTON)
+
 
 def ask(msg):
     dialog = Gtk.MessageDialog(None,
@@ -91,6 +106,7 @@ def ask(msg):
 
 DESKTOP_GROUP = GLib.KEY_FILE_DESKTOP_GROUP
 
+
 class IconPicker(object):
     def __init__(self, dialog, button, image):
         self.dialog = dialog
@@ -102,7 +118,7 @@ class IconPicker(object):
         chooser = Gtk.FileChooserDialog(title=_("Choose an icon"),
                                         parent=self.dialog,
                                         buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
-                                        Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
+                                                 Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
         chooser.add_shortcut_folder("/usr/share/pixmaps")
         chooser.add_shortcut_folder("/usr/share/icons")
         fn = get_icon_string(self.image)
@@ -115,9 +131,9 @@ class IconPicker(object):
                 icon_info_fn = icon_info.get_filename() if icon_info != None else None
                 if icon_info_fn:
                     chooser.set_filename(icon_info_fn)
-        filter = Gtk.FileFilter();
-        filter.add_pixbuf_formats ();
-        chooser.set_filter(filter);
+        filter = Gtk.FileFilter()
+        filter.add_pixbuf_formats ()
+        chooser.set_filter(filter)
 
         preview = Gtk.Image()
         chooser.set_preview_widget(preview)
@@ -139,10 +155,11 @@ class IconPicker(object):
                 preview.set_from_pixbuf(pixbuf)
                 chooser.set_preview_widget_active(True)
 
+
 class ItemEditor(object):
     ui_file = None
 
-    def __init__(self, item_path = None, callback = None, destdir = None):
+    def __init__(self, item_path=None, callback=None, destdir=None):
         self.builder = Gtk.Builder()
         self.builder.add_from_file(self.ui_file)
         self.callback = callback
@@ -262,6 +279,7 @@ class ItemEditor(object):
             self.callback(False, self.item_path)
         self.dialog.destroy()
 
+
 class LauncherEditor(ItemEditor):
     ui_file = '/usr/share/cinnamon/cinnamon-desktop-editor/launcher-editor.ui'
 
@@ -302,7 +320,7 @@ class LauncherEditor(ItemEditor):
         chooser = Gtk.FileChooserDialog(title=_("Choose a command"),
                                         parent=self.dialog,
                                         buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
-                                        Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
+                                                 Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
         response = chooser.run()
         if response == Gtk.ResponseType.ACCEPT:
             self.builder.get_object('exec-entry').set_text(escape_space(chooser.get_filename()))
@@ -398,12 +416,13 @@ class CinnamonLauncherEditor(ItemEditor):
         chooser = Gtk.FileChooserDialog(title=_("Choose a command"),
                                         parent=self.dialog,
                                         buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
-                                        Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
+                                                 Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
         response = chooser.run()
         if response == Gtk.ResponseType.ACCEPT:
             self.builder.get_object('exec-entry').set_text(escape_space(chooser.get_filename()))
         chooser.destroy()
 
+
 class Main:
     def __init__(self):
         parser = OptionParser()
@@ -419,9 +438,9 @@ class Main:
             parser.error("directory and launcher modes must be accompanied by the -o argument")
         if options.mode == "nemo-launcher" and not options.destination_directory:
             parser.error("nemo-launcher mode must be accompanied by the -d argument")
-        if options.mode == "cinnamon-launcher" and len(args) < 3:
-            parser.error("cinnamon-launcher mode must have the following syntax:\n\
-                         cinnamon-desktop-editor -mcinnamon-launcher [-ffoo.desktop] <uuid> <instance-id> <json-path>")
+        if options.mode == "cinnamon-launcher" and len(args) < 1:
+            parser.error("cinnamon-launcher mode must have the following syntax:\n"
+                         "cinnamon-desktop-editor -mcinnamon-launcher [-ffoo.desktop] <json-path>")
 
         self.tree = CMenu.Tree.new("cinnamon-applications.menu", CMenu.TreeFlags.INCLUDE_NODISPLAY)
         if not self.tree.load_sync():
@@ -433,9 +452,7 @@ class Main:
         self.dest_dir = options.destination_directory
 
         if options.mode == "cinnamon-launcher":
-            self.uuid = args[0]
-            self.iid = args[1]
-            self.json_path = args[2]
+            self.json_path = args[0]
 
         if self.desktop_file is not None:
             self.get_desktop_path()
@@ -453,7 +470,7 @@ class Main:
             editor = LauncherEditor(self.orig_file, self.nemo_launcher_cb, self.dest_dir)
             editor.dialog.show_all()
         else:
-            print "Invalid args"
+            print("Invalid args")
 
     def directory_cb(self, success, dest_path):
         self.end()
@@ -463,9 +480,8 @@ class Main:
 
     def panel_launcher_cb(self, success, dest_path):
         if success:
-            factory = XletSettingsWidgets.Factory(self.json_path, self.iid, False, self.uuid)
-
-            launchers = factory.settings.get_value("launcherList")
+            settings = JsonSettingsWidgets.JSONSettingsHandler(self.json_path)
+            launchers = settings.get_value("launcherList")
             if self.desktop_file is None:
                 launchers.append(os.path.split(dest_path)[1])
             else:
@@ -473,7 +489,7 @@ class Main:
                 if i >= 0:
                     del launchers[i]
                     launchers.insert(i, os.path.split(dest_path)[1])
-            factory.settings.set_value("launcherList", launchers)
+            settings.save_settings()
             if self.desktop_file is None:
                 self.ask_menu_launcher(dest_path)
         self.end()
@@ -516,6 +532,6 @@ class Main:
         Gtk.main_quit()
 
 if __name__ == "__main__":
-    Gtk.Window.set_default_icon_name('system-run')
+    Gtk.Window.set_default_icon_name(DEFAULT_ICON_NAME)
     Main()
     Gtk.main()
